package main

import (
	"bufio"
	"fmt"
	"github.com/BurntSushi/toml"
	"os"
	"path"
	"path/filepath"
	"runtime"
	"strings"
)

type Config struct {
	WowDir               string
	LaunchCmd            string
	WinePrefix           string
	EnableLauncher       bool
	RemoveUnknownPatches bool
}

const (
	configDirName    = "epochcli"
	configName       = "config.toml"
	defaultWowPath   = "/path/to/wow"
	defaultLaunchCmd = "not configured"
)

var (
	configRun = false
	cfgPath   string
)

func setupConfig(rerun bool) (*Config, error) {
	home, err := os.UserHomeDir()
	if err != nil {
		return nil, fmt.Errorf("unable to determine home directory: %v", err)
	}
	cfgPath = filepath.Join(home, ".config", configDirName, configName)

	newConfig := Config{
		WowDir:               defaultWowPath,
		LaunchCmd:            defaultLaunchCmd,
		EnableLauncher:       false,
		RemoveUnknownPatches: true,
	}

	_, statErr := os.Stat(cfgPath)
	if rerun || os.IsNotExist(statErr) {
		configRun = true
		fmt.Println("Enter the path to your Wow directory below. Use the full path without shortcuts like '~' (ex: /home/user/epoch):")
		s, err := readLine()
		if err != nil {
			return nil, fmt.Errorf("unable to read input: %v", err)
		}
		newConfig.WowDir = s

		fmt.Println()
		p, err := promptYesNo(fmt.Sprintf("Do you want to use epochcli to launch Wow? Select No if you plan on using a launcher tool like Lutris (y/n): "))
		if err != nil {
			return nil, err
		}
		fmt.Println()

		if p {
			newConfig.EnableLauncher = true
			newConfig.LaunchCmd = path.Join(newConfig.WowDir, "Project-Epoch.exe")

			if runtime.GOOS == "windows" {
				exePath, err := os.Executable()
				if err != nil {
					return nil, fmt.Errorf("unable to create desktop shortcut: %v", err)
				}
				err = makeLink(exePath, path.Join(home, "Desktop", "Project-Epoch.lnk"))
				if err != nil {
					return nil, fmt.Errorf("unable to create desktop shortcut: %v", err)
				}
			} else {
				s, err = input("Enter your wine prefix. Leave blank if you do not need to set WINEPREFIX")
				if err != nil {
					return nil, err
				}
				fmt.Println()

				newConfig.WinePrefix = s
				newConfig.LaunchCmd = "wine " + newConfig.LaunchCmd

				fmt.Println("Your launch command has been set to the following:")
				if s == "" {
					fmt.Printf("wine %s\n", newConfig.LaunchCmd)
				} else {
					fmt.Printf("WINEPREFIX=%s %s\n", newConfig.WinePrefix, newConfig.LaunchCmd)
				}
				fmt.Printf("Modify the configuration file at %s if you need to customize it\n\n", cfgPath)
			}
		}

		err = os.MkdirAll(filepath.Join(home, ".config", configDirName), 0755)
		if err != nil {
			return nil, fmt.Errorf("unable to create config directory: %v", err)
		}

		file, err := os.Create(cfgPath)
		if err != nil {
			file.Close()
			return nil, fmt.Errorf("unable to create config file: %v", err)
		}

		encoder := toml.NewEncoder(file)
		if err = encoder.Encode(newConfig); err != nil {
			file.Close()
			return nil, fmt.Errorf("unable to encode config file: %v", err)
		}
		file.Close()

		fmt.Printf("Created new config at %s\n\n", cfgPath)
	}

	_, err = toml.DecodeFile(cfgPath, &newConfig)
	if err != nil {
		return nil, err
	}

	// Rewrite config file with any new defaults
	file, err := os.Create(cfgPath)
	if err != nil {
		return nil, fmt.Errorf("unable to config file: %v", err)
	}
	defer file.Close()

	encoder := toml.NewEncoder(file)
	if err = encoder.Encode(newConfig); err != nil {
		return nil, fmt.Errorf("unable to encode config file: %v", err)
	}

	return &newConfig, nil
}

func promptYesNo(prompt string) (bool, error) {
	for {
		fmt.Print(prompt)
		s, err := readLine()
		if err != nil {
			return false, fmt.Errorf("unable to read input: %v", err)
		}

		if s == "y" || s == "Y" {
			return true, nil
		}

		if s == "n" || s == "N" {
			return false, nil
		}
		fmt.Println("Please enter a valid value of either 'y' or 'n'")
	}
}

func input(prompt string) (string, error) {
	fmt.Println(prompt)
	fmt.Printf("> ")

	s, err := readLine()
	if err != nil {
		return "", fmt.Errorf("unable to read input: %v", err)
	}
	return s, nil
}

func readLine() (string, error) {
	reader := bufio.NewReader(os.Stdin)
	in, err := reader.ReadString('\n')
	if err != nil {
		return "", fmt.Errorf("unable to read input: %v", err)
	}
	return strings.TrimSpace(in), nil
}
